﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using Phidget22;
using Phidget22.Events;
using Phidget22.ExampleUtils;

namespace Gyroscope_Example {
	public partial class Form1 : Form {
		CommandLineOpen open;
        formCleaner cleaner;
        Gyroscope gyroscope = null;
		private ErrorEventBox errorBox;

		double[] lastMsCount = { 0, 0, 0 };
		bool[] lastMsCountGood = { false, false, false };
		double[] gyroHeading = { 0, 0, 0 }; //degrees

		public Form1() {
			open = new CommandLineOpen(this);
            cleaner = new formCleaner(this);
            InitializeComponent();
		}

		public Form1(String[] commandLine) {
			open = new CommandLineOpen(this);
            cleaner = new formCleaner(this);
            open.commandLine = commandLine;
			InitializeComponent();
		}

		private void Form1_Load(object sender, EventArgs e) {
			Phidget.InvokeEventCallbacks = true;
			errorBox = new ErrorEventBox(this);
			gyroscope = new Gyroscope();

			//register necessary handlers
			gyroscope.Attach += gyroscope_attach;
			gyroscope.Detach += gyroscope_detach;
			gyroscope.Error += gyroscope_error;
			gyroscope.AngularRateUpdate += angularRate_update;

			//open the hardware for use
			open.openCmdLine(gyroscope);

		}

		private void Form1_FormClosing(object sender, FormClosingEventArgs e) {
			gyroscope.Close();
		}




		void gyroscope_attach(object sender, Phidget22.Events.AttachEventArgs e) {
			phidgetInfoBox1.FillPhidgetInfo((Phidget)sender);
			Gyroscope attachedDevice = (Gyroscope)sender;

			settingsBox.Visible = true;
			outputBox.Visible = true;
			gaugeBox.Visible = true;

			gyroHeading[0] = 0;
			gyroHeading[1] = 0;
			gyroHeading[2] = 0;
			xPosition.Value = 0;
			yPosition.Value = 0;
			zPosition.Value = 0;

			try {//these parameters must be set or the device won't function
				attachedDevice.DataInterval = 32; //setting the interval lower than this is possible but it can cause the GUI to bog down
                dataIntervalTrk.Unit = "ms";
				dataIntervalTrk.minTxt = attachedDevice.MinDataInterval.ToString();
				dataIntervalTrk.Minimum = attachedDevice.MinDataInterval;
				dataIntervalTrk.maxTxt = attachedDevice.MaxDataInterval.ToString();
				dataIntervalTrk.Maximum = attachedDevice.MaxDataInterval;
				dataIntervalTrk.Value = 32;
				dataIntervalTrk.trkText = attachedDevice.DataInterval.ToString();
			} catch (PhidgetException ex) {
				errorBox.addMessage("Error setting data interval: " + ex.Message);
			}
		}


		void gyroscope_detach(object sender, Phidget22.Events.DetachEventArgs e) {
			phidgetInfoBox1.Clear();
            cleaner.clean();

            settingsBox.Visible = false;
			outputBox.Visible = false;
			gaugeBox.Visible = false;
		}
		void gyroscope_error(object sender, Phidget22.Events.ErrorEventArgs e) {
			errorBox.addMessage(e.Description);
		}


		void angularRate_update(object sender, Phidget22.Events.GyroscopeAngularRateUpdateEventArgs e) {
			try {//display the angular rates and calculate the angular positions
				calculateGyroHeading(e, 0);
				calculateGyroHeading(e, 1);
				calculateGyroHeading(e, 2);

				xRatetxt.Text = e.AngularRate[0].ToString() + " °/s";
				yRatetxt.Text = e.AngularRate[1].ToString() + " °/s";
				zRatetxt.Text = e.AngularRate[2].ToString() + " °/s";
				timestamptxt.Text = e.Timestamp.ToString() + " ms";

				xPosition.Value = (float)gyroHeading[0];
				yPosition.Value = (float)gyroHeading[1];
				zPosition.Value = (float)gyroHeading[2];
			} catch (PhidgetException ex) {
				errorBox.addMessage("Error reading output: " + ex.Message);
			}
		}

		private void dataIntervalTrk_Scroll(object sender, EventArgs e) {
			if (MouseButtons != MouseButtons.Left) {
				try {
					gyroscope.DataInterval = (int)dataIntervalTrk.Value;
				} catch (Exception ex) {
					errorBox.addMessage("Error setting data interval: " + ex.Message);
				}
			}
		}

		private void dataIntervalTrk_LogMouseUp(object sender, MouseEventArgs e) {
			try {
				gyroscope.DataInterval = (int)dataIntervalTrk.Value;
			} catch (Exception ex) {
				errorBox.addMessage("Error setting data interval: " + ex.Message);
			}
		}

		void calculateGyroHeading(Phidget22.Events.GyroscopeAngularRateUpdateEventArgs data, int index) {
			double gyro = 0;
			gyro = data.AngularRate[index];

			if (lastMsCountGood[index]) {//calculate heading
				double timechange = data.Timestamp - lastMsCount[index]; // in ms
				double timeChangeSeconds = (double)timechange / 1000.0;

				gyroHeading[index] += timeChangeSeconds * gyro;

				//ensure proper wrap around
				if (gyroHeading[index] > 180)
					gyroHeading[index] -= 360;
				else if (gyroHeading[index] < -180)
					gyroHeading[index] += 360;
			}

			lastMsCount[index] = data.Timestamp;
			lastMsCountGood[index] = true;
		}

		private void zeroBtn_Click(object sender, EventArgs e) {//zero the gyroscope values
			gyroscope.Zero();
			System.Threading.Thread.Sleep(100);
			gyroHeading[0] = 0;
			gyroHeading[1] = 0;
			gyroHeading[2] = 0;
		}


	}
}